// File : DAC.c
// Description: DAC APIs
// Setup for Z8F082A, running PWM on timer0
// Run about 5 kHz PWM off of 5.5 MHz sysclock yields a 10-bit DAC

#include <ez8.h>
#include "DAC.h"

unsigned char DAC_READY;				// Flag indicates DAC has settled
unsigned int dac_interrupt_count;		// Counter for settling time

void dac_initialize(void)
{
	// Port A config for Z8F082A, setup as PWM on Timer0
	PAADDR = 0x02;		//Port A alternate function
	PACTL |= 0x02;		//Pin PA1 is timer0 out, or PWM in this case

	PAADDR = 0x04;		//Port A output control
	PAHDE |= 0x02;		//  Pin PA1 high output drive

	/******************************************************************
	* Initialize Timer 0 in continuous PWM mode with prescale = 1X
	* Initialize the PWM width registers to 1 (min. duty cycle).
	* Initialize the PWM period to 200 uSeconds (5 KHz).
	*******************************************************************/  
	T0CTL0	 = 0x60;	// Interrupt only on reload/compare events
	T0CTL1   = 0x43;    // Disabled, Polarity 1, Prescale set to X1, PWM mode
     
	T0PWMH  = 0x00;		// PWM width Hi 
 	T0PWML  = 0x01;		// PWM width Lo	

	// To set PWM freq, sysclk is divided by the value of T0CP(H,L)
	// About 5 kHz from 5.5 MHz = divide by 1025 = 0x400

	T0CPH   = 0x04;		// Reload Hi
	T0CPL   = 0x00;		// Reload Lo 

	/******************************************************************
	* Initialize Timer 0 Interrupt with high priority.
	* Point interrupt vector to void dac_isr(void);
	*******************************************************************/
	IRQ0ENH |= 0x20;	// Enable timer 0 interupts as high priority.
	IRQ0ENL |= 0x20;  	// INTR is generated as RELOAD value is reached.
	SET_VECTOR (TIMER0,dac_isr);

	dac_interrupt_count = 0;
	DAC_READY = TRUE;
}

void dac_api(int input)
{	
	DAC_READY = FALSE;

	if (input <= 0x00)	
		input = 0x01;		// Timer doesn't support 0x00; Make 0x01.
							// Tried turning off, but goes to 3.3 V out
	
	T0PWML = input & 0xFF;	// Convert the calculated PWM value 
	input = input >> 8;		// to two bytes and set the PWM hi/low
	T0PWMH = input & 0xFF;	// registers.
	
	T0CTL1 |= 0x80;			// Enable timer PWM (it may have been disabled)
}

#pragma interrupt
void interrupt dac_isr(void)
{
	if (dac_interrupt_count++ > 100)	// Calculated delay amount; DAC output is stable.
	{	
		dac_interrupt_count = 0; 		
		DAC_READY = TRUE;
	}
}
