// PID.C
// Real-time PID controller.  Updated 1-26-08.
// Translation from LV6.1 PID Toolkit.  All numbers are floats.
// Returns controller output as a function of PV, SP, and gains.
// Note scaling of variables! Must be scaled externally.

#include "PID.h"

float PID( 
	float PV,	// Process variable 0..100 percent
	float SP,	// Setpoint 0..100 percent
	float Kc, 	// Proportional gain = 0.22
	float Ki,	// Integral gain, repeats/min = 0.06
	float Kd,	// Derivative gian, repeats/min= 0.08
	float dt,	// time (min) since last call to this function
	char init	// Initialize if zero
	)
{	
	float err;				// Error in percent
	float err_last;			// Error (n-1)
	float PV1;				// PV (n-1)
	float PV2;				// PV (n-2)
	float PV3;				// PV (n-3)
	float filtered_PV;		// PV after FIR filter
	float deriv_action;		// Result of derivative algo
	float prop_action;		// Result of proportional algo
	float integral_action;	// Result of integral algo
	float integrated_error;	// Intermediate variable for integral algo
	float integrated_error_last;  // 
	float u;				// Output, 0..10
	
	// Initialize misc. variables
	if (init == 0)
	{
		PV1 = 0.0; PV2 = 0.0; PV3 = 0.0; 
		err_last = 0.0;
		integrated_error_last = 0.0;
	}
	
	err = SP - PV;
	prop_action = Kc * err;
	
	// IIR LPF on PV. Only for derivative.
	filtered_PV = PV * FIR1 + PV1 * FIR2 + PV2 * FIR3 + PV3 * FIR4;
	
	// Derivative action
	if (init == 0)
		deriv_action = 0.0;
	else
		deriv_action = (PV1 - filtered_PV) * Kd / dt;
	
	if (deriv_action > PID_MAX_DERIV_ACTION) deriv_action = PID_MAX_DERIV_ACTION;
	else if (deriv_action < PID_MIN_DERIV_ACTION) deriv_action = PID_MIN_DERIV_ACTION;
	
	// Integral action
	if (init == 0)
		integrated_error = 0.0;
	 else
		integrated_error = err * dt + integrated_error_last;
		
	 integral_action = integrated_error * Ki;	 

	 // Check controller output limits. Recalc integrated error if needed.
	 u = prop_action + integral_action + deriv_action;
	 if (  (u >= PID_OUT_MAX) || (u < PID_OUT_MIN) )
	 {
		if (u >= PID_OUT_MAX)
		{
			integral_action = PID_OUT_MAX - (prop_action + deriv_action);
			integrated_error = integral_action / Ki;
		}
		else 
		{
			integral_action = PID_OUT_MIN - (prop_action + deriv_action);
			integrated_error = integral_action / Ki;
		}
	 }
 
	 integrated_error_last = integrated_error; 
	 err_last = err;
	 PV3 = PV2;
	 PV2 = PV1;
	 PV1 = filtered_PV;
	 
	 // Controller output, limited to [PID_OUT_MIM .. PID_OUT_MAX]
	 return  (prop_action + integral_action + deriv_action);
}	
