/******************************************************************************
 * File       : adc.c 
 * Description: Initializes  Z8EncoreX on chip ADC peripheral to Continuouss
 *				conversion mode. The ISR reads the 10 bit data
 *				from ADC data register, applies error correction, and stores to latest_adc_value.
 *				Turns out ADC actually overloads at 2.16 V input.
 *
 GWJ 1-24-08. Adapted from poorly-coded Zilog example
 ************************************************************************/////
#include <ez8.h>
#include "adc.h"

// ADC data
unsigned char adc_low_byte;  	// Stores the High and Low byte ADC value  
int adc_high_byte; 	
unsigned char ADC_READY;			// Flag indicates ADC averaged reading is ready
	
extern int latest_adc_value;		// Corrected ADC value
extern unsigned int sample_count;	// Count of ADC interrupts
extern long int adc_sum;			// Running sum of ADC voltages. Div by sample_count to get Volts.
extern long int adc_avg;		// Avg of running sum of ADC voltages. 

void init_ADC(void)
{
	SET_VECTOR(ADC, isr_ADC);				// Pass the vector number and the ISR address
											// to be placed into the interrupt table
	// Setup Port B for ADC and (optional) external voltage reference
	PBADDR 	= 0x02;     					// PortB Alternate function
	PBCTL 	|= 0x01;   						// PortB bit0 as alt func = ADC0
#ifdef EXTREF
	PBCTL	|= 0x20;						// PortB bit5 as alt func = Ext Vref
#else
	PBCTL	&= ~0x20;						// PortB bit5 as alt func = Int Vref	
#endif
	
	// Setup ADC control register for desired voltage ref source
#ifdef EXTREF
	ADCCTL1 &= ~0x80;						// Ext ref select, high bit; Set for Ext
	ADCCTL0 &= ~0x20;						// Ext ref select, low bit; Set for Ext
#else
	ADCCTL1 |= 0x80;						// Ext ref select, high bit; Set for Int 2.0 V
	ADCCTL0 &= ~0x20;						// Ext ref select, low bit; Set for Int
#endif

	// Setup ADC
	//		Bits 0..3: AI chan 0
	//		Bit 4: Continous sampling, every 256 system clks.
	ADCCTL0 = 0x10;						

	// Enable ADC interrupts; nominal priority (2)
	IRQ0E1 |= 0x01;		//ADC_INT_ENH
	IRQ0E0 &= ~0x01;	//ADC_INT_ENL
	ADCCTL0 |= 0x80;	// Bit 7 (CEN): enable ADC sampling
}		


/**************************************************************************************/
/* ADC Interrupt handler
The ADC value is read after every conversion. The 10 bit ADC value is signed.
It is added to an accumulator, adc_sum, at each interrupt, and a sample counter
is also incremented. adc_sum / sample_count = average measurement at any time.
/**************************************************************************************/
#pragma interrupt
void isr_ADC(void)  
{
	IRQ0 &= 0xFE;		// Disable the pending interrupts

	// Fetch raw ADC reading
	// 11-bit twos-complement result written to {ADCD_H[7:0], ADCD_L[7:5]}.
	adc_high_byte =  ADCD_H; 
	adc_low_byte =  ADCD_L; 

	if((adc_low_byte & 0x01) == 0x01)	// Check for  ADC overflow condition
		{			
		adc_high_byte = 0x7F;	// Force ADC value to look like maximum voltage
		adc_low_byte = 0xE0;
//latest_adc_value = 0;	// Force the ADC value to zero. Do not count this sample.
		PAOUT &= ~0x80;		// Turn On LED Yellow
//		return;
		}
	else PAOUT |= 0x80;		// Turn off LED Yellow when ADC is in range
	latest_adc_value = ( adc_high_byte << 8); 	// Move to hi byte. Puts sign bit at MSb
	latest_adc_value |= (adc_low_byte & 0xE0);  // only keep top 3 bits of LSB
	latest_adc_value = latest_adc_value >> 5;	// Downshift to 10-bit resolution while keeping sign
	adc_sum += latest_adc_value;				// Add to running sum for averaging
	sample_count++;								// Count the sample
	if (sample_count >= LOOP_CYCLE_PERIOD)  // Terminal count for desired averaging period
	{
		adc_avg = adc_sum / sample_count;		// Compute average counts
		adc_sum = 0;
		sample_count = 0;						// Reset sample counter
		ADC_READY = 1;
	}
return;
} 

