// File : main.c
// Description: Oven heater controller.  GWJ 1-26-08
//  Rev. 3-38-08. Tweak constants for assembled VNA.

#include <ez8.h>
#include <sio.h>
#include <math.h>
#include "main.h"
#include "DAC.h"
#include "ADC.h"
#include "PID.h"

// Global Variables
unsigned int sample_count = 0;	// Count of ADC interrupts
int latest_adc_value = 0;		// Corrected ADC value
long int adc_sum = 0;			// Running sum of ADC voltages. Div by sample_count to get Volts.
long int adc_avg = 0;			// Avg of running sum of ADC voltages. 

extern unsigned char DAC_READY;	// Flag indicates DAC has settled
extern unsigned char ADC_READY;	// Flag indicates ADC averaged reading is ready

void main()
{
	// PID variables
   	float PV = 0.0;		// Process variable 0..100 percent
	float SP = 60.0;	// Setpoint 0..100 percent. 10% = 1 degC
	float Kc = 0.13; 	// Proportional gain = 0.13
	float Ki = 0.044;	// Integral gain, repeats/min = 0.04
	float Kd = 0.05;		// Derivative gian, repeats/min= 0.05
	float dt = 0.0083333;	// time (min) since last call to this function
							// 0.5 sec for this app
	float err;			// SP-PV to decide how to light status LED
	
	// State machine enum						
	enum stateType {Initialize, Eval_Temp, Timed_Heatup, Coasting, Untimed_Heatup, PID_Init, PID_Run};
	enum stateType state = Initialize;

	int input = 1;					// Value written to DAC
	float adc_avg_volts;			// ADC averged voltage
	float pid_out;					// Result of PID calc
	int discard_cnt = 0;			// Counter for discarding ADC samples
	int ticks = 0;				// Counter for elapsed time in ADC ticks (0.5 sec)
	
	init_systemclock();				// Initialize System clock source
	DI();							// Disable Interrupts
	init_led_gpio();				// Initializes LED ports (Port A and C)
	dac_initialize();				// Inititalize PWM DAC
	EI();							// Enable interrupts. NOTE: Must be BEFORE int_ADC()
	init_ADC();						// Initialze ADC

	dac_api(input);					// Load new input to DAC.

	while (1)
	{
		// ADC averager, driven by interrupts
		if (sample_count > 4500) LED_PC7(0);	// Turn off actiivity LED
		if (ADC_READY)  // Terminal count for desired averaging period
		 {
		 	adc_avg_volts = adc_avg * ADC_VOLTS_PER_COUNT;	// Scale to Volts
			if( adc_avg_volts > 2.5 ) adc_avg_volts = 2.7;  // Avoid bizarre values
			if( adc_avg_volts < -0.1 ) adc_avg_volts = 0.0; // Avoid bizarre values
		 	
		 	// Scale ADC voltage to PID PV percentage
			// Info: 	Thermistor amp: 1.0 V = 40 deg, -0.2 V/deg
			// 			Temp = 45 - 0.2 * adc_avg_volts
			PV = 50 * (2.5 - adc_avg_volts); 
			LED_PC7(1);		// Turn On activity Red
			ADC_READY = 0;		// Clear ADC ready flag for next loop

			switch( state )
			{
			case Initialize:	 // Discard some ADC samples
				{
					if( discard_cnt++ > 5 ) state = Eval_Temp; 
				}
				break;
				
			case Eval_Temp:		 // See where PV is, and decide what to do
				{
					if( PV > (SP-20) ) state = PID_Init;  // Close enough, no heating needed
					else if( PV > 2 )  // Set up for timed heating
					{	// See main.h for calculation 
						ticks = (SP - PV) * 1.1;
						state = Timed_Heatup;
					}	
					else state = Untimed_Heatup;  // Temp below range of PV measurement
				}
				break;		
				
			case Timed_Heatup:	// Heat a max power until ticks=0
				{
					if( ticks > 0 ) 
						{
							dac_api( DAC_MAX ); // Heater on max
							blink_led( 1 ); // Blink grn LED fast
							ticks--;
						}
					else 
						{
							dac_api( DAC_MAX / 14 ); // Heater to "cruise" current
							ticks = 20;  // 10 sec coast  3-28-08
							state = Coasting;
						}
				}
				break;
			
			case Coasting:	// After any heatup event, wait about 10 sec for settling
				{
					if( ticks-- <= 0 ) state = PID_Init;
				}
				break;
				
			case Untimed_Heatup:
				{
					if( PV < 55 )	// Looking for temp to reach middle of measurement band
									// Depending on trajectory, adjust between 2% and SP so that it hits SP nicely
						{
							dac_api( DAC_MAX ); // Heater on max
							blink_led( 1 ); // Blink grn LED fast
						}
					else
						{
							dac_api( DAC_MAX / 14 ); // Heater to "cruise" current
							ticks = 20;  // 10 sec coast  3-28-08
							state = Coasting;
						}
				}
				break;
				
			case PID_Init: 		
				{
					PID( PV, SP, Kc, Ki, Kd, dt, 0); // initialize PID algo
					ticks = 4;  // Needed to continue PID init with live data
					state = PID_Run;
				}
				break;
				
			case PID_Run:		// Do PID calc continuously	
				{
					//  Output is limited to 0..10. DAC range is 0 to 3.3 V
					pid_out = PID( PV, SP, Kc, Ki, Kd, dt, 1); 		
					input = DAC_COUNTS_PER_VOLT * pid_out / 3.3;
						
					if( ticks > 0 ) ticks--;  // Short pause to prime PID
					else
					{
						dac_api(input);	// Write to DAC output
						err = SP - PV;
						if( fabs(err) <= 0.2 )
							LED_PC6(1);	// Inside a small error band. LED on solid
						else if( err > 0.2 )
							blink_led( 2 ); // Too low. Blink grn LED fast
						else	
							blink_led( 6 ); // Too high. Blink grn LED slow
					}
				}
				break;
				default:	{}
			} //end switch
						
		 } // end if(ADC_READY)
	} // end while
} // end main

