// main.h for WB9JPS Temperature Controller
// Updated 2-1-08
// Updated 3-28-08 for full VNA

/*
	Main loop is timed by ADC acquisitions, which are calibrated to 0.5 seconds for a cycle or 'tick'.
	For each loop, process variable (PV) is scaled to percent and compared with setpoint (SP).
	A state machine helps with startup by doing a full-blast heatup if appropriate.
	
	PV calibraiton: 10 degC for 2.0 V at ADC. Scaled to a 100% range, 1% = 0.1 degC = 0.02 V.
	
	Since PV range is only 10 degC total, we have to do a balistic heatup if temp is way low. This
	is called Untimed Heatup. Then there is deadtime which yields about 6 degC of extra rise after
	heater is turned off; we allow 80 sec for that.
	
	If PV is within measurement range (6 degC or 60% below SP), then we can do a timed heatup.
	This uses knowledge of the process heat capacity, 147.5 J/degC, and heater output, 24 W.
	Running the heater for 1 tick yields 12 J, and a temp rise of 12/147.5 = 0.0813 degC, 
	or 0.813 % PV.
	
	PID is called once for initialization to set the time-dependent parts of the algorithm to 
	match process conditions. Then it is called for each tick. 
	
	There are two LEDs. One shows the control algorithm status. It blinks fast when heating,
	slow when cooling, and is on steady when inside a small deadband around the setpoint.
	The other LED simply shows that the algorithm is running and changes state each pass
	thru the main loop.
	
	ADC is run full-speed, which is about 20 kHz, and then 0.5 sec worth of data is averaged
	for each result. The DAC is PWM-based at 5 kHz and yields about 10 bits of resolution.
	
*/

/* Files required in project:
	main.c
	ADC.c
	DAC.c
	gpio.c
	pid.c
	sysclock.c
*/

#define MAIN

///////////////////////////////////////////////////////////
// Definition Program status
#define RUN			0
#define BUSY		1

// ADC calibration
//#define ADC_VOLTS_PER_COUNT 0.0020707	// for 2.0 V intenal ref
#define ADC_VOLTS_PER_COUNT 0.00267	// for 2.5 V external ref

// DAC calibration: 1024 counts for 3.3 V
#define DAC_COUNTS_PER_VOLT 315.4
#define DAC_MAX 1024

///////////////////////////////////////////////////////////
// Function Prototypes

void init_systemclock();
void init_led_gpio(void);
void LED_PC6( char state );
void LED_PC7( char state );
void blink_led( int interval );

// External clock support
/* Instructions for enabling the external clock:
 * 1. Select External clock and Frequency in the Debugger Setup Target dialog
 * 2. Uncomment the following line, rebuild, download, and run */
//#define CLOCKSOURCE    CLOCK_EXTERNAL
#ifndef CLOCKSOURCE
#define CLOCKSOURCE    CLOCK_INTERNAL
#endif

// Default user flash optionbits.
FLASH_OPTION1 = 0xFF;
FLASH_OPTION2 = 0xFF;
